(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@hpcc-js/ddl-shim'), require('@hpcc-js/dgrid'), require('@hpcc-js/phosphor'), require('@hpcc-js/codemirror-shim'), require('@hpcc-js/common')) :
	typeof define === 'function' && define.amd ? define(['exports', '@hpcc-js/ddl-shim', '@hpcc-js/dgrid', '@hpcc-js/phosphor', '@hpcc-js/codemirror-shim', '@hpcc-js/common'], factory) :
	(factory((global['@hpcc-js/codemirror'] = {}),global['@hpcc-js/ddl-shim'],global['@hpcc-js/dgrid'],global['@hpcc-js/phosphor'],global['@hpcc-js/codemirror-shim'],global['@hpcc-js/common']));
}(this, (function (exports,ddlShim,dgrid,phosphor,codemirrorShim,common) { 'use strict';

function __$styleInject(css, returnValue) {
  if (typeof document === 'undefined') {
    return returnValue;
  }
  css = css || '';
  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';
  head.appendChild(style);
  
  if (style.styleSheet){
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
  return returnValue;
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = Object.setPrototypeOf ||
    ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
    function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

__$styleInject(".codemirror_Editor {\r\n    border-style: solid;\r\n    border-width: 1px;\r\n    border-color: lightgray;\r\n}",undefined);

var Editor = /** @class */ (function (_super) {
    __extends(Editor, _super);
    function Editor() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._initialText = "";
        return _this;
    }
    Editor.prototype.options = function () {
        return {
            lineNumbers: true,
            tabSize: 2
        };
    };
    Editor.prototype.text = function (_) {
        if (this._codemirror) {
            var doc = this._codemirror.getDoc();
            if (!arguments.length)
                return doc.getValue();
            doc.setValue(_);
            return this;
        }
        if (!arguments.length)
            return this._initialText;
        this._initialText = _;
        return this;
    };
    Editor.prototype.enter = function (domNode, element) {
        var _this = this;
        _super.prototype.enter.call(this, domNode, element);
        this._codemirror = codemirrorShim.CodeMirror.fromTextArea(element.append("textarea").node(), this.options());
        this._codemirror.on("changes", function (changes) {
            _this.changes(changes);
        });
        this.text(this._initialText);
    };
    Editor.prototype.update = function (domNode, Element) {
        _super.prototype.update.call(this, domNode, Element);
        this._codemirror.setSize(this.width() - 2, this.height() - 2);
        this._codemirror.refresh();
    };
    //  Events  ---
    Editor.prototype.changes = function (changes) {
    };
    return Editor;
}(common.HTMLWidget));
Editor.prototype._class += " codemirror_Editor";

var JSONEditor = /** @class */ (function (_super) {
    __extends(JSONEditor, _super);
    function JSONEditor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    JSONEditor.prototype.options = function () {
        return __assign({}, _super.prototype.options.call(this), {
            mode: "application/json"
        });
    };
    JSONEditor.prototype.json = function (_) {
        if (!arguments.length)
            return JSON.parse(this.text());
        this.text(JSON.stringify(_, null, "\t"));
        return this;
    };
    return JSONEditor;
}(Editor));
JSONEditor.prototype._class += " codemirror_JSONEditor";

__$styleInject("",undefined);

var DDLEditor = /** @class */ (function (_super) {
    __extends(DDLEditor, _super);
    function DDLEditor() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.summary = "0 Errors";
        _this._ddlEditor = new JSONEditor().on("changes", function (changes) {
            _this.checkSyntax();
        });
        _this._errorTable = new dgrid.Table()
            .columns(["dataPath", "keyword", "message", "params"]);
        return _this;
    }
    DDLEditor.prototype.ddl = function (_) {
        if (!arguments.length)
            return this._ddlEditor.json();
        this._ddlEditor.json(_);
        this.checkSyntax();
        return this;
    };
    DDLEditor.prototype.checkSyntax = function () {
        try {
            var json = this.ddl();
            try {
                var d = ddlShim.validate2(json);
                if (d.success) {
                    this._errorTable.data([]);
                }
                else {
                    this._errorTable.data(d.errors.map(function (error) { return [error.dataPath, error.keyword, error.message, JSON.stringify(error.params)]; }));
                }
            }
            catch (e) {
                this._errorTable.data([["", "Validation Failed", "", ""]]);
            }
        }
        catch (e) {
            this._errorTable.data([["", "Invalid JSON", "", ""]]);
        }
        if (this._renderCount) {
            this._errorTable.lazyRender();
        }
    };
    DDLEditor.prototype.enter = function (domNode, element) {
        _super.prototype.enter.call(this, domNode, element);
        this
            .addWidget(this._ddlEditor)
            .addWidget(this._errorTable);
    };
    DDLEditor.prototype.update = function (domNode, _element) {
        _super.prototype.update.call(this, domNode, _element);
    };
    return DDLEditor;
}(phosphor.SplitPanel));
DDLEditor.prototype._class += " codemirror_DDLEditor";

var ECLEditor = /** @class */ (function (_super) {
    __extends(ECLEditor, _super);
    function ECLEditor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ECLEditor.prototype.options = function () {
        var retVal = _super.prototype.options.call(this);
        retVal.mode = "text/x-ecl";
        return retVal;
    };
    return ECLEditor;
}(Editor));
ECLEditor.prototype._class += " codemirror_ECLEditor";

var JSEditor = /** @class */ (function (_super) {
    __extends(JSEditor, _super);
    function JSEditor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    JSEditor.prototype.options = function () {
        return __assign({}, _super.prototype.options.call(this), {
            mode: "text/javascript"
        });
    };
    JSEditor.prototype.javascript = function (_) {
        if (!arguments.length)
            return this.text();
        this.text(_);
        return this;
    };
    return JSEditor;
}(Editor));
JSEditor.prototype._class += " codemirror_JSEditor";

exports.DDLEditor = DDLEditor;
exports.ECLEditor = ECLEditor;
exports.Editor = Editor;
exports.JSEditor = JSEditor;
exports.JSONEditor = JSONEditor;

Object.defineProperty(exports, '__esModule', { value: true });

})));
